/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app.views;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreeColumnViewerLabelProvider;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TreeDragSourceEffect;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Slider;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.kuramo.javie.api.Time;
import ch.kuramo.javie.app.ColorUtil;
import ch.kuramo.javie.app.CommandIds;
import ch.kuramo.javie.app.InjectorHolder;
import ch.kuramo.javie.app.IntervalTask;
import ch.kuramo.javie.app.PropertyUtil;
import ch.kuramo.javie.app.UIUtil;
import ch.kuramo.javie.app.actions.AVIOutputAction;
import ch.kuramo.javie.app.actions.AddEffectAction;
import ch.kuramo.javie.app.actions.AddExpressionsAction;
import ch.kuramo.javie.app.actions.KeyframeInterpolationAction;
import ch.kuramo.javie.app.actions.QTMovieOutputAction;
import ch.kuramo.javie.app.actions.RemoveKeyframesAction;
import ch.kuramo.javie.app.actions.ResetAnimatableValuesAction;
import ch.kuramo.javie.app.actions.ReverseLayerTimeAction;
import ch.kuramo.javie.app.actions.SequenceOutputAction;
import ch.kuramo.javie.app.actions.TimeRemapAction;
import ch.kuramo.javie.app.actions.WaveOutputAction;
import ch.kuramo.javie.app.player.MediaPlayer;
import ch.kuramo.javie.app.player.PlayerLinkEvent;
import ch.kuramo.javie.app.player.PlayerLinkListener;
import ch.kuramo.javie.app.project.MoveEffectsOperation;
import ch.kuramo.javie.app.project.NewCameraLayerOperation;
import ch.kuramo.javie.app.project.NewLayerFromItemOperation;
import ch.kuramo.javie.app.project.NewNullLayerOperation;
import ch.kuramo.javie.app.project.NewTextLayerOperation;
import ch.kuramo.javie.app.project.ProjectEvent;
import ch.kuramo.javie.app.project.ProjectEventHub;
import ch.kuramo.javie.app.project.ProjectListener;
import ch.kuramo.javie.app.project.ProjectManager;
import ch.kuramo.javie.app.project.RemoveEffectsOperation;
import ch.kuramo.javie.app.project.RemoveLayersOperation;
import ch.kuramo.javie.app.project.ReorderLayersOperation;
import ch.kuramo.javie.app.project.ProjectEvent.Type;
import ch.kuramo.javie.app.views.layercomp.AnimatableValueElement;
import ch.kuramo.javie.app.views.layercomp.AnimatableValueElementDelegate;
import ch.kuramo.javie.app.views.layercomp.EffectAnimatableValueElement;
import ch.kuramo.javie.app.views.layercomp.EffectElement;
import ch.kuramo.javie.app.views.layercomp.EffectsElement;
import ch.kuramo.javie.app.views.layercomp.Element;
import ch.kuramo.javie.app.views.layercomp.LayerAnimatableValueElement;
import ch.kuramo.javie.app.views.layercomp.LayerElement;
import ch.kuramo.javie.app.views.layercomp.TimelineManager;
import ch.kuramo.javie.core.AnimatableValue;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.Effect;
import ch.kuramo.javie.core.EffectDescriptor;
import ch.kuramo.javie.core.EffectableLayer;
import ch.kuramo.javie.core.Folder;
import ch.kuramo.javie.core.Interpolation;
import ch.kuramo.javie.core.Item;
import ch.kuramo.javie.core.ItemLayer;
import ch.kuramo.javie.core.Keyframe;
import ch.kuramo.javie.core.Layer;
import ch.kuramo.javie.core.LayerComposition;
import ch.kuramo.javie.core.MediaItem;
import ch.kuramo.javie.core.TransformableLayer;
import ch.kuramo.javie.core.Util;
import ch.kuramo.javie.core.services.AnimatableValueCollector;
import ch.kuramo.javie.core.services.EffectRegistry;

import com.google.inject.Inject;

public class LayerCompositionView extends ViewPart
		implements ProjectListener, PlayerLinkListener {

	public static final String ID = "ch.kuramo.javie.app.views.layerCompositionView";

	public static final int NAME_COL		= 0;
	public static final int VALUE_COL		= 1;
	public static final int MODE_COL		= 2;
	public static final int TRACKMATTE_COL	= 3;
	public static final int PARENT_COL		= 4;
	public static final int SHOWHIDE_COL	= 5;
	public static final int LABEL_COL		= 6;
	public static final int TIMELINE_COL	= 7;

	public static final String PROJECT_MANAGER = "PROJECT_MANAGER";
	public static final String LAYER_COMPOSITION = "LAYER_COMPOSITION";
	public static final String ANIMATABLE_VALUES = "ANIMATABLE_VALUES";
	public static final String TIMELINE_MANAGER = "TIMELINE_MANAGER";


	private static final Logger _logger = LoggerFactory.getLogger(LayerCompositionView.class);

	private static final boolean COCOA = SWT.getPlatform().equals("cocoa");

	private static final boolean WIN32 = SWT.getPlatform().equals("win32");

	private static final int ROW_PADDING = 4;


	private Composite _topLeftSpacer;

	private Composite _bottomLeftSpacer;

	private Composite _timelineMeter;

	private Scale _timelineScale;

	private Slider _timelineSlider;

	private TreeViewer _treeViewer;

	private Action _newCameraLayerAction;

	private Action _newNullLayerAction;

	private Action _newTextLayerAction;

	private Point _mouseDownPoint;

	private volatile Time _time;

	private IntervalTask _timelineUpdateTask;

	private IntervalTask _collectTask;

	private ProjectManager _projectManager;

	private CompositionItem _compItem;

	private LayerComposition _composition;

	private TimelineManager _timelineManager;

	@Inject
	private EffectRegistry _effectRegistry;

	@Inject
	private AnimatableValueCollector _collector;


	public LayerCompositionView() {
		super();
		InjectorHolder.getInjector().injectMembers(this);

		_newCameraLayerAction = new Action("カメラ") {
			public void run() {
				handleNewCameraLayerAction();
			}
		};

		_newNullLayerAction = new Action("ヌルオブジェクト") {
			public void run() {
				handleNewNullLayerAction();
			}
		};

		_newTextLayerAction = new Action("テキスト") {
			public void run() {
				handleNewTextLayerAction();
			}
		};

		_collector.activate();
	}

	public void createPartControl(final Composite parent) {
		ProjectManager pm = ProjectManager.forWorkbenchWindow(getSite().getWorkbenchWindow());
		if (pm == null) {
			throw new IllegalStateException("No ProjectManager exists for this window.");
		}

		parent.setLayout(new FormLayout());

		_topLeftSpacer = new Composite(parent, SWT.NONE);
		_bottomLeftSpacer = new Composite(parent, SWT.NONE);

		createTreeViewer(parent);
		createTimelineControls(parent);
		setLayoutData();

		setupGlobalActionsAndContextMenu();
		setupDragAndDrop();

		_timelineUpdateTask = new IntervalTask(parent.getDisplay(), 50) {
			public void run() {
				if (!parent.isDisposed() && _timelineManager != null) {
					_timelineManager.update(_time, true);
				}
			}
		};

		_collectTask = new IntervalTask(parent.getDisplay(), 300) {
			public void run() {
				if (!parent.isDisposed()) {
					collectAnimatableValues(_time);
				}
			}
		};

		setProjectManager(pm);

		ProjectEventHub.getHub().addProjectListener(this);
	}

	public void dispose() {
		ProjectEventHub.getHub().removeProjectListener(this);

		_collector.deactivate();

		super.dispose();
	}

	public void setFocus() {
		_treeViewer.getControl().setFocus();
	}

	private String getItemId() {
		return getViewSite().getSecondaryId();
	}

	private void setProjectManager(ProjectManager pm) {
		_compItem = pm.getProject().getItem(getItemId());
		if (_compItem == null) {
			throw new IllegalStateException("no such CompositionItem found: " + getItemId());
		}

		_projectManager = pm;
		_composition = (LayerComposition) _compItem.getComposition();


		setPartName(_compItem.getName());

		_treeViewer.setData(PROJECT_MANAGER, pm);
		_treeViewer.setData(LAYER_COMPOSITION, _composition);
		_treeViewer.setInput(_composition);

		_time = Time.fromFrameNumber(0, _composition.getFrameDuration());
		_timelineManager = new TimelineManager(
				pm, _composition, this, _timelineMeter, _timelineScale, _timelineSlider, _treeViewer.getTree());
		_treeViewer.setData(TIMELINE_MANAGER, _timelineManager);

		collectAnimatableValues();
		connectToPlayer();
	}

	public void handleProjectEvent(ProjectEvent event) {
		ProjectManager pm = event.getProjectManager();
		if (pm.getWorkbenchWindow() != getSite().getWorkbenchWindow()) {
			return;
		}

		switch (event.type) {
			case LAYER_PROPERTY_CHANGE:
			case LAYERS_REMOVE:
			case EFFECT_PROPERTY_CHANGE:
			case EFFECTS_REMOVE:
				_timelineManager.clearKeyframeSelection();
				_timelineManager.redraw();
				break;
		}

		switch (event.type) {
			case ITEM_NAME_CHANGE:
				handleItemNameChange(event);
				break;

			case LAYER_PROPERTY_CHANGE:
				handleLayerPropertyChange(event);
				break;

			case LAYER_EXPRESSION_CHANGE:
				handleLayerExpressionChange(event);
				break;

			case LAYERS_ADD:
			case LAYERS_REMOVE:
			case LAYERS_REORDER:
				handleLayersAddRemoveReorder(event);
				break;

			case EXPRESSIONS_ADD:
			case EXPRESSIONS_REMOVE:
				handleExpressionsAddRemove(event);
				break;

			case EFFECTS_ADD:
			case EFFECTS_REMOVE:
				handleEffectsAddRemove(event);
				break;

			case EFFECT_PROPERTY_CHANGE:
				handleEffectPropertyChange(event);
				break;

			case EFFECT_EXPRESSION_CHANGE:
				handleEffectExpressionChange(event);
				break;

			case LAYER_TIMES_CHANGE:
				handleLayerTimesChange(event);
				break;

			case KEYFRAMES_CHANGE:
				handleKeyframesChange(event);
				break;

			case LAYER_SLIP_EDIT:
				handleLayerSlipEdit(event);
				break;
		}
	}

	private void createTimelineControls(Composite parent) {
		_timelineMeter = new Composite(parent, WIN32 ? SWT.DOUBLE_BUFFERED : SWT.NONE);
		_timelineScale = new Scale(parent, SWT.HORIZONTAL);
		_timelineSlider = new Slider(parent, SWT.HORIZONTAL);

		SelectionListener selectionListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				_timelineManager.update(_time, false);
			}
		};

		_timelineScale.addSelectionListener(selectionListener);
		_timelineSlider.addSelectionListener(selectionListener);
	}

	private void createTreeViewer(Composite parent) {
		class DragSourceEffect extends TreeDragSourceEffect {
			private boolean creatingImage;

			public DragSourceEffect(Tree tree) {
				super(tree);
			}

			public void dragStart(DragSourceEvent event) {
				creatingImage = true;
				try {
					super.dragStart(event);
				} finally {
					creatingImage = false;
				}
			}
		}

		_treeViewer = new TreeViewer(parent, SWT.MULTI | SWT.NO_SCROLL | SWT.V_SCROLL);
		_treeViewer.setUseHashlookup(true);
		_treeViewer.setContentProvider(new LayerCompositionContentProvider());

		Tree tree = _treeViewer.getTree();
		tree.setHeaderVisible(true);
		tree.setLinesVisible(!COCOA);

		final DragSourceEffect dragEffect = new DragSourceEffect(tree);
		tree.setData("DEFAULT_DRAG_SOURCE_EFFECT", dragEffect);

		createViewerColumn(NAME_COL,      200, null, "ソース名");
		createViewerColumn(VALUE_COL,     150, null, "");
		createViewerColumn(MODE_COL,       50, null, "モード");
		createViewerColumn(TRACKMATTE_COL, 50, null, "ﾄﾗｯｸﾏｯﾄ");
		createViewerColumn(PARENT_COL,    100, null, "親");
		createViewerColumn(SHOWHIDE_COL,   60, null, "");
		createViewerColumn(LABEL_COL,      40, null, "");
		createViewerColumn(TIMELINE_COL,  300, null, "");


		ControlAdapter columnWidthAdjuster = new ControlAdapter() {
			private boolean adjusting = false;
			public void controlResized(ControlEvent e) {
				if (!adjusting) {
					adjusting = true;
					try {
						adjustColumnWidth();

						if (_timelineManager != null) {
							_timelineManager.update(_time, false);
						}
					} finally {
						adjusting = false;
					}
				}
			}
		};

		tree.addControlListener(columnWidthAdjuster);

		for (TreeColumn column : tree.getColumns()) {
			column.addControlListener(columnWidthAdjuster);
		}


		tree.setBackground(ColorUtil.tableBackground());

		if (COCOA) {
			final Color layerColor = ColorUtil.layerRowBackground();
			final Color lineColor = ColorUtil.tableRowLine();

			tree.addListener(SWT.EraseItem, new Listener() {
				public void handleEvent(Event event) {
					if (dragEffect.creatingImage) {
						return;
					}

					GC gc = event.gc;

					TreeItem treeItem = (TreeItem) event.item;
					Object element = treeItem.getData();
					if (element instanceof LayerElement) {
						gc.setBackground(layerColor);
						gc.fillRectangle(event.x, event.y, event.width, event.height - 1);
					}

					int x = event.x;
					int y = event.y + event.height - 1;
					gc.setForeground(lineColor);
					gc.drawLine(x, y, x + event.width, y);
				}
			});

		} else {
			final Color layerColor = ColorUtil.layerRowBackground();

			// SWT.SetData イベントで TreeItem#setBackground して背景色を設定しようとすると、
			// レイヤーの順序を入れ替えたときに背景色が正しく設定されないことがあるようなので、
			// SWT.EraseItem イベント時に塗りつぶす。
			tree.addListener(SWT.EraseItem, new Listener() {
				public void handleEvent(Event event) {
					if (dragEffect.creatingImage) {
						// WIN32ではここですぐに抜けても特に変化は無いようではあるが、とりあえず抜けておく。
						return;
					}

					TreeItem treeItem = (TreeItem) event.item;
					Object element = treeItem.getData();
					if (element instanceof LayerElement) {
						GC gc = event.gc;
						gc.setBackground(layerColor);
						gc.fillRectangle(event.x, event.y, event.width, event.height);

						// TreeItemの背景色を設定しておかないと、
						// 選択時のみカラムテキストの背景がツリーの背景色になってしまう。
						treeItem.setBackground(layerColor);
					}
				}
			});
		}

		tree.addListener(SWT.PaintItem, new Listener() {
			public void handleEvent(Event event) {
				Object element = event.item.getData();
				if (element instanceof Element) {
					((Element) element).paintColumn(event);
				}
			}
		});

		tree.addListener(SWT.MeasureItem, new Listener() {
			public void handleEvent(Event e) {
				e.height = Math.max(e.height, e.gc.getFontMetrics().getHeight() + ROW_PADDING*2);
			}
		});

		tree.addMouseMoveListener(new MouseMoveListener() {
			public void mouseMove(MouseEvent e) {
				updateCursor(e);
			}
		});

		tree.addMouseListener(new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				if (COCOA) {
					_mouseDownPoint = new Point(e.x, e.y);
				}
				handleMouseDown(e);
			}
		});

		tree.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				handleKeyPressed(e);
			}
		});

		tree.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				if (_timelineManager != null) {
					_timelineManager.drawTimeIndicatorLine(e);
					if (COCOA) {
						_timelineManager.drawColumnLeftLine(e);
					}
				}
			}
		});
	}

	private TreeViewerColumn createViewerColumn(int columnIndex, int width, Image image, String name) {
		TreeViewerColumn viewerColumn = new TreeViewerColumn(_treeViewer, SWT.NONE);
		TreeColumn column = viewerColumn.getColumn();
		column.setWidth(width);
		column.setImage(image);
		column.setText(name);
		column.setMoveable(false);

		viewerColumn.setLabelProvider(new TreeColumnViewerLabelProvider(new LayerCompositionLabelProvider()));
		viewerColumn.setEditingSupport(createEditingSupport(columnIndex));

		return viewerColumn;
	}

	private EditingSupport createEditingSupport(final int columnIndex) {
		return new EditingSupport(_treeViewer) {

			protected boolean canEdit(Object element) {
				return (element instanceof Element) ? ((Element) element).canEdit(columnIndex) : false;
			}

			protected CellEditor getCellEditor(Object element) {
				return (element instanceof Element) ? ((Element) element).getCellEditor(columnIndex) : null;
			}
			
			protected Object getValue(Object element) {
				return (element instanceof Element) ? ((Element) element).getCellEditorValue(columnIndex) : null;
			}

			protected void setValue(Object element, Object value) {
				if (element instanceof Element) {
					((Element) element).setCellEditorValue(columnIndex, value);
				}
			}
		};
	}

	private void setupGlobalActionsAndContextMenu() {
		// GlobalActions

		IViewSite site = getViewSite();
		IActionBars actionBars = site.getActionBars();
		IWorkbenchWindow window = site.getWorkbenchWindow();

		actionBars.setGlobalActionHandler(CommandIds.NEW_CAMERA_LAYER, _newCameraLayerAction);
		actionBars.setGlobalActionHandler(CommandIds.NEW_NULL_LAYER, _newNullLayerAction);
		actionBars.setGlobalActionHandler(CommandIds.NEW_TEXT_LAYER, _newTextLayerAction);
		if (AVIOutputAction.isAvailable()) {
			actionBars.setGlobalActionHandler(CommandIds.AVI_OUTPUT, new AVIOutputAction(this));
		}
		if (QTMovieOutputAction.isAvailable()) {
			actionBars.setGlobalActionHandler(CommandIds.QTMOVIE_OUTPUT, new QTMovieOutputAction(this));
		}
		actionBars.setGlobalActionHandler(CommandIds.SEQUENCE_OUTPUT, new SequenceOutputAction(this));
		actionBars.setGlobalActionHandler(CommandIds.WAVE_OUTPUT, new WaveOutputAction(this));

		List<AddEffectAction> effectActions = Util.newList();
		for (EffectDescriptor ed : _effectRegistry.getEffectDescriptors()) {
			AddEffectAction action = new AddEffectAction(window, _treeViewer, ed);
			actionBars.setGlobalActionHandler("EFFECT." + ed.getType(), action);
			effectActions.add(action);
		}


		// ContextMenu

		MenuManager menuMgr = new MenuManager();
		MenuManager newMenu = new MenuManager("新規");
		MenuManager timeMenu = new MenuManager("時間");
		MenuManager effectMenu = new MenuManager("エフェクト");
		MenuManager animeMenu = new MenuManager("アニメーション");
		MenuManager interpolationMenu = new MenuManager("キーフレーム補間法");

		menuMgr.add(newMenu);
		menuMgr.add(timeMenu);
		menuMgr.add(effectMenu);
		menuMgr.add(animeMenu);
		menuMgr.add(new Separator());
		menuMgr.add(new ResetAnimatableValuesAction(window, _treeViewer));
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

		newMenu.add(_newCameraLayerAction);
		newMenu.add(_newNullLayerAction);
		newMenu.add(_newTextLayerAction);
		timeMenu.add(new TimeRemapAction(window, _treeViewer));
		timeMenu.add(new ReverseLayerTimeAction(_treeViewer));
		setupEffectContextMenu(effectMenu, effectActions);
		animeMenu.add(interpolationMenu);
		animeMenu.add(new RemoveKeyframesAction(_treeViewer));
		animeMenu.add(new AddExpressionsAction(window, _treeViewer));
		interpolationMenu.add(new KeyframeInterpolationAction(_treeViewer, Interpolation.HOLD));
		interpolationMenu.add(new KeyframeInterpolationAction(_treeViewer, Interpolation.LINEAR));
		interpolationMenu.add(new KeyframeInterpolationAction(_treeViewer, Interpolation.CATMULL_ROM));

		getSite().registerContextMenu(menuMgr, _treeViewer);

		Control treeViewControl = _treeViewer.getControl();
		Menu treeViewMenu = menuMgr.createContextMenu(treeViewControl);
		treeViewControl.setMenu(treeViewMenu);
	}

	private void setupEffectContextMenu(MenuManager effectMenu, List<AddEffectAction> effectActions) {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint("ch.kuramo.javie.api.effectCategory");
		if (point == null) {
			return;
		}

		Map<String, MenuManager> subMenus = Util.newMap();

		for (IExtension ext : point.getExtensions()) {
			for (IConfigurationElement cfgElem : ext.getConfigurationElements()) {
				if ("effect-category".equals(cfgElem.getName())) {
					String category = cfgElem.getAttribute("category");
					String name = cfgElem.getAttribute("name");

					MenuManager subMenu = new MenuManager(name);
					subMenus.put(category, subMenu);
					effectMenu.add(subMenu);
				}
			}
		}

		for (AddEffectAction action : effectActions) {
			MenuManager subMenu = subMenus.get(action.getEffectDescriptor().getCategory());
			if (subMenu != null) {
				subMenu.add(action);
			} else {
				effectMenu.add(action);
			}
		}
	}

	private void setupDragAndDrop() {
		final LocalSelectionTransfer selectionTransfer = LocalSelectionTransfer.getTransfer();

		DragSourceListener sourceListener = new DragSourceListener() {
			public void dragStart(DragSourceEvent event) {
				if (!canDrag(event)) {
					event.doit = false;
					return;
				}

				ISelection selection = new LocalSelectionWrapper(_treeViewer);
				if (selection.isEmpty()) {
					event.doit = false;
				} else {
					selectionTransfer.setSelection(selection);
					selectionTransfer.setSelectionSetTime(event.time & 0xFFFFFFFFL);
					event.doit = true;
				}
			}

			public void dragSetData(DragSourceEvent event) {
				event.data = selectionTransfer.getSelection();
			}

			public void dragFinished(DragSourceEvent event) {
				selectionTransfer.setSelection(null);
				selectionTransfer.setSelectionSetTime(0);
			}
		};

		DropTargetListener targetListener = new DropTargetAdapter() {

			private <T extends Element> T getTargetElement(DropTargetEvent event, Class<T> elementClass) {
				TreeItem treeItem = (TreeItem) event.item;
				if (treeItem != null) {
					Object data = treeItem.getData();
					if (elementClass.isInstance(data)) {
						@SuppressWarnings("unchecked")
						T element = (T) data;
						return element;
					}
				}
				return null;
			}

			private List<LayerElement> getSourceLayerElements(DropTargetEvent event) {
				List<LayerElement> list = Util.newList();
				ITreeSelection selection = LocalSelectionWrapper.getSelection(selectionTransfer, _treeViewer);
				if (selection != null) {
					for (Object o : selection.toList()) {
						if (o instanceof LayerElement) {
							list.add((LayerElement) o);
						} else {
							return null;
						}
					}
				}
				return list.isEmpty() ? null : list;
			}

			private List<Item> getSourceItems(DropTargetEvent event) {
				List<Item> list = Util.newList();
				ITreeSelection selection = LocalSelectionWrapper.getSelection(selectionTransfer);
				if (selection != null) {
					for (Object o : selection.toList()) {
						if (o instanceof Folder) {
							// TODO Folderを再帰的に調べる。
						}
						if (o instanceof MediaItem) {
							list.add((MediaItem) o);
						} else {
							return null;
						}
					}
				}
				return list.isEmpty() ? null : list;
			}

			private List<EffectElement> getSourceEffectElements(DropTargetEvent event) {
				List<EffectElement> list = Util.newList();
				ITreeSelection selection = LocalSelectionWrapper.getSelection(selectionTransfer, _treeViewer);
				if (selection != null) {
					Layer layer = null;
					for (Object o : selection.toList()) {
						if (o instanceof EffectElement) {
							EffectElement ee = (EffectElement) o;
							LayerElement le = (LayerElement) ee.parent.parent;
							if (layer == null) {
								layer = le.layer;
							} else if (le.layer != layer) {
								return null;
							}
							list.add(ee);
						} else {
							return null;
						}
					}
				}
				return list.isEmpty() ? null : list;
			}

			public void dragOver(DropTargetEvent event) {
				if (!selectionTransfer.isSupportedType(event.currentDataType)) {
					event.detail = DND.DROP_NONE;
					event.feedback = DND.FEEDBACK_NONE;
					return;
				}

				if (getSourceLayerElements(event) != null) {
					if (event.item != null && getTargetElement(event, LayerElement.class) == null) {
						event.detail = DND.DROP_NONE;
						event.feedback = DND.FEEDBACK_NONE;
					} else {
						event.detail = event.operations & DND.DROP_MOVE;
						event.feedback = (event.feedback | DND.FEEDBACK_INSERT_BEFORE)
									& ~DND.FEEDBACK_SELECT & ~DND.FEEDBACK_EXPAND;
					}
					return;
				}

				if (getSourceItems(event) != null) {
					if (event.item != null && getTargetElement(event, LayerElement.class) == null) {
						event.detail = DND.DROP_NONE;
						event.feedback = DND.FEEDBACK_NONE;
					} else {
						event.detail = event.operations & DND.DROP_LINK;
						event.feedback = (event.feedback | DND.FEEDBACK_INSERT_BEFORE)
									& ~DND.FEEDBACK_SELECT & ~DND.FEEDBACK_EXPAND;
					}
					return;
				}

				if (getSourceEffectElements(event) != null && event.item != null) {
					if (getTargetElement(event, EffectsElement.class) != null) {
						event.detail = ((event.operations & DND.DROP_MOVE) == DND.DROP_MOVE)
								? DND.DROP_MOVE : (event.operations & DND.DROP_COPY);
						event.feedback = (event.feedback | DND.FEEDBACK_SELECT)
								& ~DND.FEEDBACK_EXPAND;

					} else if (getTargetElement(event, EffectElement.class) != null) {
						event.detail = ((event.operations & DND.DROP_MOVE) == DND.DROP_MOVE)
								? DND.DROP_MOVE : (event.operations & DND.DROP_COPY);
						event.feedback = (event.feedback | DND.FEEDBACK_INSERT_BEFORE)
								& ~DND.FEEDBACK_SELECT & ~DND.FEEDBACK_EXPAND;

					} else {
						event.detail = DND.DROP_NONE;
						event.feedback = DND.FEEDBACK_NONE;
					}
					return;
				}

				event.detail = DND.DROP_NONE;
				event.feedback = DND.FEEDBACK_NONE;
			}

			public void dropAccept(DropTargetEvent event) {
				dragOver(event);
			}

			public void drop(DropTargetEvent event) {
				List<LayerElement> sourceLayerElements = getSourceLayerElements(event);
				if (sourceLayerElements != null) {
					moveLayers(sourceLayerElements, getTargetElement(event, LayerElement.class));
					return;
				}

				List<Item> sourceItems = getSourceItems(event);
				if (sourceItems != null) {
					newLayers(sourceItems, getTargetElement(event, LayerElement.class));
					return;
				}

				List<EffectElement> sourceEffectElements = getSourceEffectElements(event);
				if (sourceEffectElements != null) {
					int targetIndex = -1;
					EffectsElement effectsElement = getTargetElement(event, EffectsElement.class);
					if (effectsElement == null) {
						EffectElement ee = getTargetElement(event, EffectElement.class);
						if (ee != null) {
							effectsElement = (EffectsElement) ee.parent;
							LayerElement le = (LayerElement) effectsElement.parent;
							targetIndex = ((EffectableLayer) le.layer).getEffects().indexOf(ee.effect);
						}
					}
					if (effectsElement != null) {
						EffectableLayer dstLayer = (EffectableLayer) ((LayerElement) effectsElement.parent).layer;
						if (event.detail == DND.DROP_COPY) {
							//copyEffects(sourceEffectElements, dstLayer, targetIndex);
						} else {
							moveEffects(sourceEffectElements, dstLayer, targetIndex);
						}
						return;
					}
				}

				event.detail = DND.DROP_NONE;
			}
		};

		_treeViewer.addDragSupport(DND.DROP_COPY | DND.DROP_MOVE, new Transfer[] { selectionTransfer }, sourceListener);
		_treeViewer.addDropSupport(DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK, new Transfer[] { selectionTransfer }, targetListener);
	}

	private void setLayoutData() {
		FormData data = new FormData();
		data.left = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		data.top = new FormAttachment(0, 12);
		data.bottom = new FormAttachment(0, 30);
		_topLeftSpacer.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		data.top = new FormAttachment(100, COCOA ? -15 : -16);
		data.bottom = new FormAttachment(100, 0);
		_bottomLeftSpacer.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(_topLeftSpacer, 0, SWT.RIGHT);
		data.right = new FormAttachment(100, 0);
		data.top = new FormAttachment(0, 12);
		data.bottom = new FormAttachment(0, 30);
		_timelineMeter.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(_bottomLeftSpacer, 0, SWT.RIGHT);
		data.right = new FormAttachment(_bottomLeftSpacer, 150, SWT.RIGHT);
		data.top = new FormAttachment(100, COCOA ? -15 : -27);
		data.bottom = new FormAttachment(100, 0);
		_timelineScale.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(_timelineScale, 0, SWT.RIGHT);
		data.right = new FormAttachment(100, COCOA ? -15 : -16);
		data.top = new FormAttachment(100, COCOA ? -15 : -16);
		data.bottom = new FormAttachment(100, 0);
		_timelineSlider.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		data.top = new FormAttachment(_topLeftSpacer, 0, SWT.BOTTOM);
		data.bottom = new FormAttachment(_timelineSlider, 0, SWT.TOP);
		_treeViewer.getTree().setLayoutData(data);
	}

	private void adjustColumnWidth() {
		Tree tree = _treeViewer.getTree();

		int clientWidth = tree.getClientArea().width;
		if (clientWidth == 0) {
			return;
		}

		TreeColumn[] columns = tree.getColumns();
		int totalColumnWidth = 0;
		for (TreeColumn column : columns) {
			totalColumnWidth += column.getWidth();
		}

		TreeColumn lastColumn = columns[columns.length-1];
		int lastColumnWidth = lastColumn.getWidth() + clientWidth - totalColumnWidth;
		if (lastColumnWidth > 0) {
			lastColumn.setWidth(lastColumnWidth);
		}

		FormData data1 = (FormData) _topLeftSpacer.getLayoutData();
		FormData data2 = (FormData) _bottomLeftSpacer.getLayoutData();
		data1.right.numerator = data2.right.numerator = 0;
		data1.right.offset = data2.right.offset = clientWidth - lastColumnWidth;

		final Composite parent = _bottomLeftSpacer.getParent();
		parent.getDisplay().asyncExec(new Runnable() {
			public void run() {
				if (!parent.isDisposed()) {
					parent.layout(new Control[] { _topLeftSpacer, _bottomLeftSpacer });
				}
			}
		});
	}

	private void updateCursor(MouseEvent e) {
		Point pt = new Point(e.x, e.y);
		TreeItem item = getItem(pt);
		if (item != null) {
			Object element = item.getData();
			if (element instanceof Element) {
				for (int i = 0, cc = _treeViewer.getTree().getColumnCount(); i < cc; ++i) {
					if (item.getBounds(i).contains(pt)) {
						((Element) element).updateCursor(e, i);
						return;
					}
				}
			}
		}
		_treeViewer.getTree().setCursor(null);
	}

	private TreeItem _shiftSelectionBaseItem;

	private void emulateFullSelectionMouseDown(MouseEvent e, TreeItem item, int columnIndex) {
		final Tree tree = _treeViewer.getTree();
		Collection<TreeItem> treeItems = null;

		if ((e.stateMask & SWT.MOD1) != 0) {
			treeItems = Util.newSet(Arrays.asList(tree.getSelection()));
			if (treeItems.contains(item)) {
				treeItems.remove(item);
			} else {
				treeItems.add(item);
			}
			_shiftSelectionBaseItem = item;

		} else if ((e.stateMask & SWT.MOD2) != 0 && _shiftSelectionBaseItem != null) {
			List<TreeItem> list = new Object() {
				final List<TreeItem> items = Util.newList();
				List<TreeItem> getVisibleExpandedItems() {
					getVisibleExpandedItems(tree.getItems());
					return items;
				}
				void getVisibleExpandedItems(TreeItem[] treeItems) {
					for (TreeItem ti : treeItems) {
						items.add(ti);
						if (ti.getExpanded()) {
							getVisibleExpandedItems(ti.getItems());
						}
					}
				}
			}.getVisibleExpandedItems();

			int i1 = list.indexOf(_shiftSelectionBaseItem);
			int i2 = list.indexOf(item);
			if (i1 != -1 && i2 != -1) {
				treeItems = list.subList(Math.min(i1, i2), Math.max(i1, i2)+1);
			}
		} else if (columnIndex == TIMELINE_COL
				&& (e.stateMask & SWT.BUTTON_MASK) == 0
				&& Arrays.asList(tree.getSelection()).contains(item)) {

			// フルセレクションをエミュレートするだけならこれは不要だが、
			// デュレーションバー上でのドラッグなどを適切に処理するためには
			// ここで抜けたほうが都合が良い。
			return;
		}

		if (treeItems == null) {
			treeItems = Collections.singleton(item);
			_shiftSelectionBaseItem = item;
		}

		if (columnIndex != NAME_COL || !item.getBounds().contains(new Point(e.x, e.y))) {
			selectAndReveal(treeItems);
		}
	}

	private void handleMouseDown(MouseEvent e) {
		Point pt = new Point(e.x, e.y);
		TreeItem item = getItem(pt);
		if (item != null) {
			Object element = item.getData();
			if (element instanceof Element) {
				for (int i = 0, cc = _treeViewer.getTree().getColumnCount(); i < cc; ++i) {
					if (item.getBounds(i).contains(pt)) {
						if (WIN32) {
							emulateFullSelectionMouseDown(e, item, i);
						}
						((Element) element).mouseDown(e, i);
						return;
					}
				}
			}
		}
		_timelineManager.mouseDown(e);
	}

	private void handleKeyPressed(KeyEvent e) {
		if (e.stateMask == 0 && (e.keyCode == SWT.DEL || (COCOA && e.keyCode == SWT.BS))) {
			removeSelectedObjects();
		}
	}

	private void removeSelectedObjects() {
		if (_timelineManager.hasKeyframeSelection()) {
			_timelineManager.removeSelectedKeyframes();
			return;
		}

		TreeSelection selection = (TreeSelection) _treeViewer.getSelection();
		if (selection.isEmpty()) {
			return;
		}

		List<LayerElement> layerElements = Util.newList();
		List<EffectElement> effectElements = Util.newList();
		for (Object o : selection.toList()) {
			if (o instanceof LayerElement) {
				layerElements.add((LayerElement) o);
			} else if (o instanceof EffectElement) {
				effectElements.add((EffectElement) o);
			} else if (o instanceof EffectsElement) {
				for (Element child : ((EffectsElement) o).getChildren()) {
					effectElements.add((EffectElement) child);
				}
			}
		}

		if (effectElements.size() > 0) {
			List<Object[]> effects = Util.newList();
			for (EffectElement ee : effectElements) {
				effects.add(new Object[] { ((LayerElement) ee.parent.parent).layer, ee.effect });
			}
			_projectManager.postOperation(new RemoveEffectsOperation(_projectManager, effects));

		} else if (layerElements.size() > 0) {
			List<Layer> layers = Util.newList();
			for (LayerElement le : layerElements) {
				layers.add(le.layer);
			}
			Collections.reverse(layers); // UI上での表示順とデータの並び順は逆なので。
			_projectManager.postOperation(new RemoveLayersOperation(_projectManager, layers));
		}
	}

	private boolean canDrag(DragSourceEvent e) {
		// CocoaではDragSourceEventのx,yの値がマウスをクリックした座標ではなく
		// ドラッグ検出時の座標になるようなので、mouseDownイベントで取得しておいた座標を使う。
		Point pt = (COCOA && _mouseDownPoint != null) ? _mouseDownPoint : new Point(e.x, e.y);
		TreeItem item = getItem(pt);
		if (item == null || !item.getBounds(NAME_COL).contains(pt)) {
			return false;
		}


		// 選択中の行が全てレイヤー、もしくは全てエフェクトの場合のみドラッグ可
		// さらに、エフェクトの場合は全てが同じレイヤーのエフェクトの場合のみドラッグ可。

		TreeSelection selection = (TreeSelection) _treeViewer.getSelection();
		if (selection.isEmpty()) {
			return false;
		}

		Object first = selection.getFirstElement();
		if (first instanceof LayerElement) {
			for (Object o : selection.toList()) {
				if (!(o instanceof LayerElement)) return false;
			}
			return true;

		} else if (first instanceof EffectElement) {
			Element parent = ((Element) first).parent;
			for (Object o : selection.toList()) {
				if (!(o instanceof EffectElement)) return false;
				if (((Element) o).parent != parent) return false;
			}
			return true;

		} else {
			return false;
		}
	}

	private TreeItem getItem(Point pt) {
		return getItem(pt, _treeViewer.getTree().getItems());
	}

	private TreeItem getItem(Point pt, TreeItem[] items) {
		// Tree#getItem(Point) はユーザーのクリックによって選択が生じるポイントでしかアイテムを返さないため、
		// TreeItemを順番にスキャンする必要がある。(Cocoaの場合は常にFULL_SELECTIONなので、Tree#getItem(Point)でもOK)
		// また、TreeItem#getBounds() も選択が生じる領域を返すようである(Windowsのみ？)。
		// 
		// なお、各階層において末尾のアイテムから先頭に向かって処理する方が無駄なスキャンをしなくて済む。

		for (int i = items.length-1; i >= 0; --i) {
			TreeItem item = items[i];
			Rectangle bounds = item.getBounds();
			if (bounds.y <= pt.y) {
				if (bounds.y + bounds.height > pt.y) {
					return item;
				} else if (item.getExpanded()) {
					return getItem(pt, item.getItems());
				}
			}
		}
		return null;
	}

	private void moveLayers(List<LayerElement> layerElements, LayerElement moveBefore) {
		List<LayerElement> current = Util.newList();
		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			current.add((LayerElement) treeItem.getData());
		}

		int moveToIndex = (moveBefore != null) ? current.indexOf(moveBefore) : current.size();

		// サブリストを並行して更新すると ConcurrentModificationException が発生するので複製して使う。
		List<LayerElement> frontSub = Util.newList(current.subList(0, moveToIndex));
		List<LayerElement> backSub = Util.newList(current.subList(moveToIndex, current.size()));

		frontSub.removeAll(layerElements);
		backSub.removeAll(layerElements);

		List<LayerElement> newOrderOfElements = Util.newList();
		newOrderOfElements.addAll(frontSub);
		newOrderOfElements.addAll(layerElements);
		newOrderOfElements.addAll(backSub);

		// Treeの表示順は内部データと逆順。
		Collections.reverse(newOrderOfElements);

		List<Layer> newOrder = Util.newList();
		for (LayerElement e : newOrderOfElements) {
			newOrder.add(e.layer);
		}

		List<Layer> layers = Util.newList();
		for (LayerElement e : layerElements) {
			layers.add(e.layer);
		}

		_projectManager.postOperation(new ReorderLayersOperation(_projectManager, newOrder, layers));
	}

	private void newLayers(List<Item> items, LayerElement insertBefore) {
		TreeItem[] treeItems = _treeViewer.getTree().getItems();
		int insertIndex = -1;

		if (insertBefore == null) {
			insertIndex = treeItems.length;
		} else {
			for (int i = 0; i < treeItems.length; ++i) {
				if (treeItems[i].getData() == insertBefore) {
					insertIndex = i;
					break;
				}
			}
			if (insertIndex == -1) {
				throw new IllegalArgumentException("no such layerElement found");
			}
		}

		// Treeの表示順は内部データと逆順。
		insertIndex = treeItems.length - insertIndex;

		items = Util.newList(items);
		Collections.reverse(items);

		_projectManager.postOperation(new NewLayerFromItemOperation(_projectManager, _composition, insertIndex, items));
	}

	private void moveEffects(List<EffectElement> effectElements, EffectableLayer dstLayer, int moveBefore) {
		LayerElement le = (LayerElement) effectElements.get(0).parent.parent;
		EffectableLayer srcLayer = (EffectableLayer) le.layer;
		List<Effect> effects = Util.newList();

		for (EffectElement ee : effectElements) {
			if (((LayerElement) ee.parent.parent).layer != srcLayer) {
				throw new IllegalArgumentException();
			}
			effects.add(ee.effect);
		}

		_projectManager.postOperation(new MoveEffectsOperation(_projectManager, srcLayer, effects, dstLayer, moveBefore));
	}

	private void collectAnimatableValues() {
		collectAnimatableValues(_timelineManager.getCurrentTime());
	}

	private void collectAnimatableValues(Time time) {
		_collector.reset();
		_collector.setTime(time);

		// TODO prepareExpressionはプロジェクトに構造的な変更があった場合のみ行えばよい。
		_composition.prepareExpression(_collector.createInitialExpressionScope(_composition));

		_treeViewer.setData(ANIMATABLE_VALUES, _collector.collect());
		updateProperties();
	}

	private void updateProperties() {
		updateProperties(_treeViewer.getTree().getItems());
	}

	private void updateProperties(TreeItem[] treeItems) {
		for (TreeItem item : treeItems) {
			Object element = item.getData();
			if (element instanceof AnimatableValueElement) {
				AnimatableValueElement avalueElement = (AnimatableValueElement) element;
				if (avalueElement.updateValue()) {
					Rectangle bounds = item.getBounds(VALUE_COL);
					_treeViewer.getTree().redraw(bounds.x, bounds.y, bounds.width, bounds.height, false);
				}
			}
			if (item.getExpanded()) {
				updateProperties(item.getItems());
			}
		}
	}

	private TreeItem findLayerAnimatableValueItem(TreeItem layerItem, final String property) {
		return new Object() {
			TreeItem find(TreeItem treeItem) {
				Object element = treeItem.getData();
				if (element instanceof LayerAnimatableValueElement
						&& ((LayerAnimatableValueElement) element).getProperty().equals(property)) {
					return treeItem;

				} else if (treeItem.getItemCount() > 0) {
					boolean expanded = _treeViewer.getExpandedState(element);
					_treeViewer.getTree().setRedraw(false);
					try {
						_treeViewer.setExpandedState(element, true);

						for (TreeItem ti : treeItem.getItems()) {
							ti = find(ti);
							if (ti != null) {
								return ti;
							}
						}
						return null;

					} finally {
						_treeViewer.setExpandedState(element, expanded);
						_treeViewer.getTree().setRedraw(true);
					}
				} else {
					return null;
				}
			}
		}.find(layerItem);
	}

	private TreeItem findEffectAnimatableValueItem(TreeItem layerItem, final Effect effect, final String property) {
		return new Object() {
			TreeItem find(TreeItem treeItem) {
				Object element = treeItem.getData();
				if (element instanceof EffectAnimatableValueElement
						&& ((EffectAnimatableValueElement) element).getProperty().equals(property)) {
					return treeItem;

				} else if ((element instanceof LayerElement
						|| element instanceof EffectsElement
						|| element instanceof EffectElement) && treeItem.getItemCount() > 0) {

					if (element instanceof EffectElement && ((EffectElement) element).effect != effect) {
						return null;
					}

					boolean expanded = _treeViewer.getExpandedState(element);
					_treeViewer.getTree().setRedraw(false);
					try {
						_treeViewer.setExpandedState(element, true);

						for (TreeItem ti : treeItem.getItems()) {
							ti = find(ti);
							if (ti != null) {
								return ti;
							}
						}
						return null;

					} finally {
						_treeViewer.setExpandedState(element, expanded);
						_treeViewer.getTree().setRedraw(true);
					}
				} else {
					return null;
				}
			}
		}.find(layerItem);
	}

	private TreeItem findEffectItem(TreeItem layerItem, final Effect effect) {
		return new Object() {
			TreeItem find(TreeItem treeItem) {
				Object element = treeItem.getData();
				if (element instanceof EffectElement) {
					return (((EffectElement) element).effect == effect) ? treeItem : null;

				} else if ((element instanceof LayerElement
						|| element instanceof EffectsElement) && treeItem.getItemCount() > 0) {

					boolean expanded = _treeViewer.getExpandedState(element);
					_treeViewer.getTree().setRedraw(false);
					try {
						_treeViewer.setExpandedState(element, true);

						for (TreeItem ti : treeItem.getItems()) {
							ti = find(ti);
							if (ti != null) {
								return ti;
							}
						}
						return null;

					} finally {
						_treeViewer.setExpandedState(element, expanded);
						_treeViewer.getTree().setRedraw(true);
					}
				} else {
					return null;
				}
			}
		}.find(layerItem);
	}

	private void selectAndReveal(TreeItem treeItem) {
		if (treeItem == null) {
			_treeViewer.setSelection(null);
			return;
		}

		selectAndReveal(Collections.singleton(treeItem));
	}

	private void selectAndReveal(Collection<TreeItem> treeItems) {
		if (treeItems == null || treeItems.isEmpty()) {
			_treeViewer.setSelection(null);
			return;
		}

		List<TreePath> list = Util.newList();
		for (TreeItem treeItem : treeItems) {
			LinkedList<Object> segmentList = Util.newLinkedList();
			do {
				segmentList.addFirst(treeItem.getData());
			} while ((treeItem = treeItem.getParentItem()) != null);
			list.add(new TreePath(segmentList.toArray()));
		}
		_treeViewer.setSelection(new TreeSelection(list.toArray(new TreePath[list.size()])), true);
	}

	private void handleLayerPropertyChange(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		Tree tree = _treeViewer.getTree();

		for (TreeItem treeItem : tree.getItems()) {
			LayerElement layerElement = (LayerElement) treeItem.getData();
			if (layerElement.layer == event.layer) {
				if (ProjectEvent.propertyIsAnimatable(event)) {
					treeItem = findLayerAnimatableValueItem(treeItem, event.property);

				} else {
					if (layerElement.layer instanceof TransformableLayer && event.property.equals("threeD")) {
						_treeViewer.refresh(layerElement, true);
					} else {
						_treeViewer.update(layerElement, null);
					}

					// 表示／非表示のスイッチなどは、なぜか redraw をしないと再描画されない。上の update だけじゃダメなの？
					// しかも win32 だと TreeItem#getBounds() が返す Rectangle ではダメだし...

					Rectangle bounds = treeItem.getBounds();
					if (WIN32) {
						Rectangle clientArea = tree.getClientArea();
						bounds.x = clientArea.x;
						bounds.width = clientArea.width;
					}
					tree.redraw(bounds.x, bounds.y, bounds.width, bounds.height, false);
				}

				selectAndReveal(treeItem);
				break;
			}
		}

		collectAnimatableValues();
	}

	private void handleLayerExpressionChange(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement layerElement = (LayerElement) treeItem.getData();
			if (layerElement.layer == event.layer) {
				treeItem = findLayerAnimatableValueItem(treeItem, event.property);
				if (treeItem != null) {
					Element element = (Element) treeItem.getData();

					// これをしなくても見た目はかわならいが、しないとundo/redo時にswtがクラッシュすることがある？
					_treeViewer.setExpandedState(element, false);

					_treeViewer.refresh(element, false);
					if (treeItem.getItemCount() > 0) {
						_treeViewer.setExpandedState(element, true);
						treeItem = treeItem.getItem(0);
					}
					selectAndReveal(treeItem);
				}
				break;
			}
		}

		collectAnimatableValues();
	}

	private void handleLayersAddRemoveReorder(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		_treeViewer.getTree().setRedraw(false);
		try {
			// NOTE この時点ではモデルとTreeの内容が一致していないので、
			//      refreshの前にgetExpandedElementsを呼ぶのは安全ではないかもしれない。
			Object[] expandedElements = _treeViewer.getExpandedElements();
			_treeViewer.refresh();
			_treeViewer.setExpandedElements(expandedElements);

			if (event.type == Type.LAYERS_ADD || event.type == Type.LAYERS_REORDER) {
				Set<Layer> layers = ProjectEvent.getLayers(event);
				Set<TreeItem> treeItems = Util.newSet();
				for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
					LayerElement layerElement = (LayerElement) treeItem.getData();
					if (layers.contains(layerElement.layer)) {
						treeItems.add(treeItem);
					}
				}
				selectAndReveal(treeItems);
			}

		} finally {
			_treeViewer.getTree().setRedraw(true);
		}

		collectAnimatableValues();
	}

	private void handleItemNameChange(ProjectEvent event) {
		if (event.getProjectManager() != _projectManager) {
			return;
		}

		Item item = ProjectEvent.getItem(event);
		Set<Element> elements = Util.newSet();

		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement element = (LayerElement) treeItem.getData();

			if (element.layer instanceof ItemLayer) {
				ItemLayer itemLayer = (ItemLayer) element.layer;
				if (itemLayer.getItem() == item) {
					elements.add(element);
				}
			}

			if (element.layer.getParent() instanceof ItemLayer) {
				ItemLayer parentLayer = (ItemLayer) element.layer.getParent();
				if (parentLayer.getItem() == item) {
					elements.add(element);
				}
			}
		}

		if (!elements.isEmpty()) {
			_treeViewer.update(elements.toArray(), null);
		}
	}

	private void handleExpressionsAddRemove(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		boolean add = (event.type == Type.EXPRESSIONS_ADD);

		Map<Layer, TreeItem> layerToTreeItem = Util.newMap();
		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement element = (LayerElement) treeItem.getData();
			layerToTreeItem.put(element.layer, treeItem);
		}

		Set<TreeItem> treeItems = Util.newSet();

		for (Object[] o : (Object[][]) event.data) {
			TreeItem treeItem = layerToTreeItem.get(o[0]);

			if (o[1] instanceof String) {
				String property = (String) o[1];
				treeItem = findLayerAnimatableValueItem(treeItem, property);
			} else if (o[1] instanceof Integer) {
				EffectableLayer layer = (EffectableLayer) o[0];
				Effect effect = layer.getEffects().get((Integer) o[1]);
				String property = (String) o[2];
				treeItem = findEffectAnimatableValueItem(treeItem, effect, property);
			} else {
				// never reach here
				throw new Error();
			}

			if (treeItem == null) {
				// never reach here ???
				continue;
			}

			// handleLayerExpressionChange, handleEffectExpressionChange で
			// Elementの初期化は行われているので、ここでは TreeView#refresh など何もせずに
			// treeItem.getItem(0) を呼び出して大丈夫なはず。
			if (add) {
				treeItem = treeItem.getItem(0);
			}
			treeItems.add(treeItem);
		}

		selectAndReveal(treeItems);
	}

	private void handleEffectsAddRemove(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		// 現在の選択状態を先に解除した方が見栄えがいい。解除しなくても機能的には問題ない。
		selectAndReveal((TreeItem) null);


		boolean add = (event.type == Type.EFFECTS_ADD);

		Map<Layer, TreeItem> layerToTreeItem = Util.newMap();
		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement element = (LayerElement) treeItem.getData();
			layerToTreeItem.put(element.layer, treeItem);
		}

		Set<TreeItem> treeItems = Util.newSet();

		for (Object[] o : (Object[][]) event.data) {
			TreeItem treeItem = layerToTreeItem.get(o[0]);
			LayerElement element = (LayerElement) treeItem.getData();
			Integer effectIndex = (Integer) o[1];

			_treeViewer.refresh(element);
			_treeViewer.setExpandedState(element, true);

			// 削除後に（エフェクトがひとつもなくなり）EffectsElement が存在しない場合は LayerElement を選択する。
			TreeItem itemToSelect = treeItem;

			for (TreeItem child : treeItem.getItems()) {
				if (!(child.getData() instanceof EffectsElement)) {
					continue;
				}

				if (add) {
					// 追加後は追加したエフェクトを選択する。
					_treeViewer.setExpandedState(child.getData(), true);
					itemToSelect = child.getItem(effectIndex);
				} else {
					// 削除後は（ひとつ以上エフェクトが残っていれば）EffectsElement を選択する。
					itemToSelect = child;
				}
				break;
			}

			treeItems.add(itemToSelect);
		}

		selectAndReveal(treeItems);
		collectAnimatableValues();
	}

	private void handleEffectPropertyChange(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		EffectableLayer layer = (EffectableLayer) event.layer;
		Effect effect = layer.getEffects().get(event.effectIndex);

		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement layerElement = (LayerElement) treeItem.getData();
			if (layerElement.layer == layer) {
				if (ProjectEvent.propertyIsAnimatable(event)) {
					treeItem = findEffectAnimatableValueItem(treeItem, effect, event.property);

				} else if (event.property.equals("enabled")) {
					treeItem = findEffectItem(treeItem, effect);
					// findEffectItemの中で Tree#setRedraw(false/true) をしてるので update をする必要はない。 
					//_treeViewer.update(treeItem.getData(), null);

				} else {
					// TODO AnimatableValue でなく enabled でもないプロパティの場合。
					//treeItem = findEffectPropertyItem(treeItem, effect, event.property);
				}

				selectAndReveal(treeItem);
				break;
			}
		}

		collectAnimatableValues();
	}

	private void handleEffectExpressionChange(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		EffectableLayer layer = (EffectableLayer) event.layer;
		Effect effect = layer.getEffects().get(event.effectIndex);

		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement layerElement = (LayerElement) treeItem.getData();
			if (layerElement.layer == layer) {
				treeItem = findEffectAnimatableValueItem(treeItem, effect, event.property);
				Element element = (Element) treeItem.getData();

				// これをしなくても見た目はかわならいが、しないとundo/redo時にswtがクラッシュすることがある？
				_treeViewer.setExpandedState(element, false);

				_treeViewer.refresh(element, false);
				if (treeItem.getItemCount() > 0) {
					_treeViewer.setExpandedState(element, true);
					treeItem = treeItem.getItem(0);
				}
				selectAndReveal(treeItem);
				break;
			}
		}

		collectAnimatableValues();
	}

	private void handleLayerTimesChange(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		Tree tree = _treeViewer.getTree();

		Map<Layer, TreeItem> layerToTreeItem = Util.newMap();
		for (TreeItem treeItem : tree.getItems()) {
			LayerElement element = (LayerElement) treeItem.getData();
			layerToTreeItem.put(element.layer, treeItem);
		}

		Set<TreeItem> treeItems = Util.newSet();

		@SuppressWarnings("unchecked")
		Collection<? extends Layer> layers = (Collection<? extends Layer>) event.data;
		for (Layer layer : layers) {
			TreeItem treeItem = layerToTreeItem.get(layer);
			treeItems.add(treeItem);
			//_treeViewer.update(treeItem.getData(), null);
		}

		_timelineManager.redraw();
		selectAndReveal(treeItems);
		collectAnimatableValues();
	}

	private void handleKeyframesChange(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		Map<Layer, TreeItem> layerToTreeItem = Util.newMap();
		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
			LayerElement element = (LayerElement) treeItem.getData();
			layerToTreeItem.put(element.layer, treeItem);
		}

		Set<TreeItem> treeItems = Util.newSet();
		_timelineManager.clearKeyframeSelection();

		Object[][] data = (Object[][]) event.data;

		for (Object[] o : data) {
			Layer layer = (Layer) o[0];
			int effectIndex = (Integer) o[1];
			String property = (String) o[2];
			Time time = (Time) o[3];

			TreeItem layerItem = layerToTreeItem.get(layer);
			TreeItem treeItem;
			AnimatableValue<?> avalue;

			if (effectIndex == -1) {
				treeItem = findLayerAnimatableValueItem(layerItem, property);
				avalue = PropertyUtil.getProperty(layer, property);

				if (property.equals("timeRemap")) {
					if (treeItem == null && avalue.hasKeyframe()) {
						_treeViewer.refresh(layerItem.getData(), false);
						treeItem = findLayerAnimatableValueItem(layerItem, property);
					} else if (treeItem != null && !avalue.hasKeyframe()) {
						_treeViewer.refresh(layerItem.getData(), false);
						treeItem = null;
						treeItems.add(layerItem);
					}
				}

			} else {
				Effect effect = ((EffectableLayer) layer).getEffects().get(effectIndex);
				treeItem = findEffectAnimatableValueItem(layerItem, effect, property);
				avalue = PropertyUtil.getProperty(effect, property);
			}

			if (treeItem != null) {
				treeItems.add(treeItem);

				AnimatableValueElement element = (AnimatableValueElement) treeItem.getData();
				_treeViewer.update(element, null);

				if (time != null) {
					Keyframe<?> kf = avalue.getKeyframe(time);
					_timelineManager.addKeyframeSelection(kf, element);
				}
			}
		}

		selectAndReveal(treeItems);
		collectAnimatableValues();
	}

	private void handleLayerSlipEdit(ProjectEvent event) {
		if (event.composition != _composition) {
			return;
		}

		Tree tree = _treeViewer.getTree();

		Map<Layer, TreeItem> layerToTreeItem = Util.newMap();
		for (TreeItem treeItem : tree.getItems()) {
			LayerElement element = (LayerElement) treeItem.getData();
			layerToTreeItem.put(element.layer, treeItem);
		}

		Set<TreeItem> treeItems = Util.newSet();

		@SuppressWarnings("unchecked")
		Collection<? extends Layer> layers = (Collection<? extends Layer>) ((Object[]) event.data)[0];
		for (Layer layer : layers) {
			TreeItem treeItem = layerToTreeItem.get(layer);
			treeItems.add(treeItem);

			_treeViewer.update(treeItem.getData(), null);

			// なぜか redraw をしないと再描画されない。上の update だけじゃダメなの？
			// しかも win32 だと TreeItem#getBounds() が返す Rectangle ではダメだし...

			Rectangle bounds = treeItem.getBounds();
			if (WIN32) {
				Rectangle clientArea = tree.getClientArea();
				bounds.x = clientArea.x;
				bounds.width = clientArea.width;
			}
			tree.redraw(bounds.x, bounds.y, bounds.width, bounds.height, false);
		}

	
		_timelineManager.clearKeyframeSelection();

		Object[][] data = (Object[][]) ((Object[]) event.data)[1];

		for (Object[] o : data) {
			Layer layer = (Layer) o[0];
			int effectIndex = (Integer) o[1];
			String property = (String) o[2];
			Time time = (Time) o[3];

			TreeItem treeItem;
			AnimatableValue<?> avalue;

			if (effectIndex == -1) {
				treeItem = findLayerAnimatableValueItem(layerToTreeItem.get(layer), property);
				avalue = PropertyUtil.getProperty(layer, property);
			} else {
				Effect effect = ((EffectableLayer) layer).getEffects().get(effectIndex);
				treeItem = findEffectAnimatableValueItem(layerToTreeItem.get(layer), effect, property);
				avalue = PropertyUtil.getProperty(effect, property);
			}

			treeItems.add(treeItem);

			AnimatableValueElement element = (AnimatableValueElement) treeItem.getData();
			_treeViewer.update(element, null);

			if (time != null) {
				Keyframe<?> kf = avalue.getKeyframe(time);
				_timelineManager.addKeyframeSelection(kf, element);
			}
		}


		selectAndReveal(treeItems);
		collectAnimatableValues();
	}

	private void handleNewCameraLayerAction() {
		if (_projectManager != null && _composition != null) {
			// TODO カメラオプションの設定Wizardを作る
			_projectManager.postOperation(new NewCameraLayerOperation(_projectManager, _composition));
		}
	}

	private void handleNewNullLayerAction() {
		if (_projectManager != null && _composition != null) {
			_projectManager.postOperation(new NewNullLayerOperation(_projectManager, _composition));
		}
	}

	private void handleNewTextLayerAction() {
		if (_projectManager != null && _composition != null) {
			_projectManager.postOperation(new NewTextLayerOperation(_projectManager, _composition));
		}
	}

	public CompositionItem getCompositionItem() {
		return _compItem;
	}

	public LayerComposition getLayerComposition() {
		return _composition;
	}

	public void editExpression(AnimatableValueElement avalueElement) {
		_treeViewer.setExpandedState(avalueElement, true);

		AnimatableValueElementDelegate<?>.ExpressionElement exprElement
				= (AnimatableValueElementDelegate<?>.ExpressionElement) avalueElement.getChildren()[0];

		_treeViewer.editElement(exprElement, LayerCompositionView.TIMELINE_COL);
	}

	public void update(Time time) {
		if (_playerLinkListeners.isEmpty()) {
			_time = time;
			collectAnimatableValues(time);
			_timelineManager.update(time, true);
		} else {
			collectAnimatableValues(time);
			refreshPlayer(time);
		}
	}


	private final ListenerList _playerLinkListeners = new ListenerList();

	public void addPlayerLinkListener(PlayerLinkListener listener) {
		_playerLinkListeners.add(listener);
	}

	public void removePlayerLinkListener(PlayerLinkListener listener) {
		_playerLinkListeners.remove(listener);
	}

	private void refreshPlayer(Time time) {
		PlayerLinkEvent event = new PlayerLinkEvent(this, time);
		for (Object l : _playerLinkListeners.getListeners()) {
			((PlayerLinkListener) l).handlePlayerLinkEvent(event);
		}
	}

	public void handlePlayerLinkEvent(PlayerLinkEvent event) {
		_time = event.time;
		_timelineUpdateTask.schedule();
		_collectTask.schedule();
	}


	private static final Set<String> _connectingIds = Util.newSet();

	private void connectToPlayer() {
		String itemId = getItemId();
		if (!_connectingIds.contains(itemId)) {
			_connectingIds.add(itemId);
			try {

				MediaPlayerView playerView = UIUtil.findView(getSite().getPage(), MediaPlayerView.ID, itemId);
				connectToPlayer(this, playerView);

			} finally {
				_connectingIds.remove(itemId);
			}
		}
	}

	static void connectToPlayer(MediaPlayerView playerView) {
		String itemId = playerView.getItemId();
		if (!_connectingIds.contains(itemId)) {
			_connectingIds.add(itemId);
			try {

				LayerCompositionView compView = UIUtil.findView(
						playerView.getSite().getPage(), LayerCompositionView.ID, itemId);
				connectToPlayer(compView, playerView);

			} finally {
				_connectingIds.remove(itemId);
			}
		}
	}

	private static void connectToPlayer(final LayerCompositionView compView, final MediaPlayerView playerView) {
		if (compView == null || playerView == null) {
			return;
		}

		final MediaPlayer player = playerView.getMediaPlayer();

		compView.addPlayerLinkListener(player);
		player.addPlayerLinkListener(compView);

		final IWorkbenchPage page = compView.getSite().getPage();
		IPartListener partListener = new IPartListener() {
			public void partClosed(IWorkbenchPart part) {
				if (part == playerView || part == compView) {
					compView.removePlayerLinkListener(player);
					player.removePlayerLinkListener(compView);
					page.removePartListener(this);
				}
			}

			public void partActivated(IWorkbenchPart part) {
				if (part == compView) {
					showView(playerView, compView);
				} else if (part == playerView) {
					showView(compView, playerView);
				}
			}

			private void showView(IViewPart targetView, IViewPart companionView) {
				for (IViewPart vp : page.getViewStack(companionView)) {
					if (vp == targetView) {
						return;
					}
				}
				IViewPart[] stack = page.getViewStack(targetView);
				if (stack == null) {
					return;
				}
				for (IViewPart vp : stack) {
					if (vp != targetView && page.isPartVisible(vp) &&
							vp instanceof MediaPlayerView && ((MediaPlayerView) vp).isPinned()) {
						return;
					}
				}
				try {
					IViewSite viewSite = targetView.getViewSite();
					page.showView(viewSite.getId(), viewSite.getSecondaryId(), IWorkbenchPage.VIEW_VISIBLE);
				} catch (PartInitException e) {
					_logger.error("error activating ViewPart", e);
				}
			}

			public void partOpened(IWorkbenchPart part) { }
			public void partDeactivated(IWorkbenchPart part) { }
			public void partBroughtToTop(IWorkbenchPart part) { }
		};
		page.addPartListener(partListener);
	}

}
